vehicleParts = {
	chassis = {},
	turrets = {},
	guns = {
		["gsh23"] = {
			name = "GSh-23L",
			caliber = 23,
			barrels = {
				[1] = {x = 0.05, y = 0.15, z = -0.15},
				[2] = {x = 0.25, y = 0.15, z = -0.15},
			},
			magazines = {
				[1] = {
					name = "OFZ 23mm HE"
				},
				[2] = {
					name = "BR 23mm AP",
					maxPenDepth = 0.5
				}
			},
			sight = {
				[1] = {
					x = 3.25,
					y = 1.65,
					z = 1.0
				}
			},
		},
		["rocket_mi24"] = {
			elevationSpeed = 0.5,
			sight = {
				[1] = {
					x = 3.25,
					y = 1.7,
					z = 1.7
				}
			}
		},
		["shturm"] = {
			name = "Wingtip Pylon",
			caliber = 130,
			barrels = {
				[1] = {x = 0.05, y = 0.05, z = -0.1},
				[2] = {x = 0.35, y = 0.05, z = -0.1},
			},
			magazines = {
				[1] = {
					name = "9K113 Shturm-V",
					magazineCapacity = 1,
					ammoCount = 0,
					magazineCount = 8,
					maxPenDepth = 0.5,
					explosionSize = 2.0,
				}
			},
			zoomSight = "MOD/gfx/chopperScope.png",
			sight = {
				[1] = {
					x = 3.25,
					y = 1.7,
					z = 1.7
				}
			},
		},
		["autocannon"] = {
			barrels = {
				[1] = {x = 0.25, y = 0.2, z = -0.1}
			},
			sight = {
				[1] = {
					x = 2.95,
					y = 1.5,
					z = 5.0
				}
			},
			smokeFactor = 0.5,
			zoomSight = "MOD/gfx/chopperScope.png"
		},
		["rocket_ka50"] = {
			elevationSpeed = 0.5,
			sight = {
				[1] = {
					x = 2.95,
					y = 1.5,
					z = 5.0
				}
			}
		},
		["vikhr"] = {
			name = "APU-6 Rack",
			caliber = 130,
			magazines = {
				[1] = {
					name = "9K121 Vikhr",
					magazineCapacity = 1,
					ammoCount = 0,
					magazineCount = 6,
					maxPenDepth = 0.5,
					explosionSize = 1.5,
				}
			},
			sight = {
				[1] = {
					x = 2.95,
					y = 1.5,
					z = 5.0
				}
			},
			zoomSight = "MOD/gfx/chopperScope.png"
		},
		["m230"] = {
			elevationSpeed = 0.5,
			barrels = {
				[1] = {x = 0.3, y = 0.45, z = -0.2}
			},
			sight = {
				[1] = {
					x = 2.85,
					y = 2.8,
					z = 1.5
				}
			}
		},
		["m261"] = {
			elevationSpeed = 0.5,
			sight = {
				[1] = {
					x = 2.85,
					y = 2.8,
					z = 1.5
				}
			}
		},
		["hellfire"] = {
			elevationSpeed = 0.5,
			sight = {
				[1] = {
					x = 2.85,
					y = 2.8,
					z = 1.5
				}
			}
		},
		["custom_dshk"] = {
			magazines = {
				[1] = {
					name = "12.7mm AP",
					maxPenDepth = 1.0,
					magazineCount = 10,
				}
			},
			barrels = {
				[1] = {x = 0.35, y = 0.45, z = -0.2}
			},
			sight = {
				[1] = {
					x = 0,
					y = 2.0,
					z = 1.8
				}
			}
		}
	}
}

vehicle = {}

function init()
	local sceneVehicle = FindVehicle("cfg")
	local value = GetTagValue(sceneVehicle, "cfg")
	if value == "vehicle" then
		vehicle.id = sceneVehicle

		local status, retVal = pcall(initVehicle)
		if status then
		else
			DebugPrint(retVal)
		end
	end

	SetTag(sceneVehicle, "AVF_Custom", "unset")
end

function initVehicle()
	if unexpected_condition then
		error()
	end
	vehicle.body = GetVehicleBody(vehicle.id)
	vehicle.transform = GetBodyTransform(vehicle.body)
	vehicle.shapes = GetBodyShapes(vehicle.body)
	local totalShapes = ""
	for i = 1, #vehicle.shapes do
		local value = GetTagValue(vehicle.shapes[i], "component")
		if value ~= "" then
			if value == "chassis" then
				for key, val in pairs(vehicleParts.chassis) do
					if HasTag(vehicle.shapes[i], key) then
						addItems(vehicle.shapes[i], val)
					end
				end
			end
			totalShapes = totalShapes .. value .. " "
			local test = GetShapeJoints(vehicle.shapes[i])
			for j = 1, #test do
				local val2 = GetTagValue(test[j], "component")
				if val2 ~= "" then
					totalShapes = totalShapes .. val2 .. " "

					if val2 == "turretJoint" then
						totalShapes = totalShapes .. traverseTurret(test[j], vehicle.shapes[i])
					elseif val2 == "gunJoint" then
						totalShapes = totalShapes .. addGun(test[j], vehicle.shapes[i])
					end
				end
			end
		end
	end
end

function traverseTurret(turretJoint, attatchedShape)
	local outString = ""
	local turret = GetJointOtherShape(turretJoint, attatchedShape)
	local joints = GetShapeJoints(turret)

	for j = 1, #joints do
		if joints[j] ~= turretJoint then
			local val2 = GetTagValue(joints[j], "component")

			if val2 == "turretJoint" then
				totalShapes = totalShapes .. traverseTurret(joints[j], turret)
			elseif val2 == "gunJoint" then
				outString = outString .. addGun(joints[j], turret)
			end
		end
	end
	for key, val in pairs(vehicleParts.turrets) do
		if HasTag(turret, val) then
			addItems(turret, key)
		end
	end
	return outString
end

function addGun(gunJoint, attatchedShape)
	local gun = GetJointOtherShape(gunJoint, attatchedShape)
	for key, val in pairs(vehicleParts.guns) do
		if HasTag(gun, key) then
			addItems(gun, val)
		end
	end
	local val3 = GetTagValue(gun, "component")
	return val3
end

function addItems(shape, values)
	for key, val in pairs(values) do
		if type(val) == "table" then
			SetTag(shape, "@" .. key)
			for subKey, subVal in pairs(val) do
				if type(subVal) == "table" then
					for subKey2, subVal2 in pairs(subVal) do
						if key == "magazines" then
							SetTag(shape, "@" .. string.sub(key, 1, -2) .. subKey .. "_" .. subKey2, subVal2)
						else
							SetTag(shape, "@" .. key .. subKey .. subKey2, subVal2)
						end
					end
				else
					if key == "magazines" then
						SetTag(shape, "@" .. string.sub(key, 1, -2) .. "_" .. subKey, subVal)
					else
						SetTag(shape, "@" .. key .. subKey, subVal)
					end
				end
			end
		else
			SetTag(shape, "@" .. key, val)
		end
	end
end
